require "securenet/version"
require "rest_client"
require "json"

require "securenet/util"

module SecureNet

	def self.init(id, key, url = "https://gwapi.demo.securenet.com/api")
		@util = SecurenetUtil.new(id, key, url)
	end

	################################################################################
	# Batch Routines
	################################################################################

	def self.getCurrentBatch
		return @util.get("/batches/Current", nil)
	end

	def self.getBatch(batchid)
		return @util.get("/batches/#{batchid}", nil)
	end

	def self.closeBatch
		return @util.post("/batches/Close", nil)
	end

	################################################################################
	# Transaction Processing
	################################################################################

	def self.authorize(data)
		return @util.post("/Payments/Authorize", data)
	end

	def self.capture(data)
		return @util.post("/Payments/Capture", data)
	end

	def self.charge(data)
		return @util.post("/Payments/Charge", data)
	end

	################################################################################
	# Vault
	################################################################################

	def self.createCustomer(data)
		return @util.post("/Customers", data)
	end

	def self.getCustomer(data)
		if data.class == Hash
			if ! data.has_key?("customerId".to_sym)
				return @util.error("customerId is required")
			end
			customerId = data[:customerId]
		else
			customerId = data
		end
		return @util.get("/Customers/#{customerId}", nil)
	end

	def self.updateCustomer(data)
		if ! data.has_key?("customerId".to_sym)
			return @util.error("customerId is required")
		end
		return @util.put("/Customers/#{data[:customerId]}", data)
	end


	def self.createCustomerPaymentMethod(data)
		if ! data.has_key?("customerId".to_sym)
			return @util.error("customerId is required")
		end
		return @util.post("/Customers/#{customerId}", data)
	end

	def self.getCustomerPaymentMethod(data)
		if ! data.has_key?("customerId".to_sym)
			return @util.error("customerId is required")
		end
		if ! data.has_key?("paymentMethodId".to_sym)
			return @util.error("paymentMethodId is required")
		end
		return @util.get("/Customers/#{data[:customerId]}/PaymentMethod/#{data[:paymentMethodId]}", data)
	end

	def self.deleteCustomerPaymentMethod(data)
		if ! data.has_key?("customerId".to_sym)
			return @util.error("customerId is required")
		end
		if ! data.has_key?("paymentMethodId".to_sym)
			return @util.error("paymentMethodId is required")
		end
		return @util.delete("/Customers/#{data[:customerId]}/PaymentMethod/#{data[:paymentMethodId]}", data)
	end

	################################################################################
	# Installment Plans
	################################################################################

	def self.createInstallmentPlan(data)
		if ! data.has_key?("customerId".to_sym)
			return @util.error("customerId is required")
		end
		return @util.post("/Customers/#{customerId}/PaymentSchedules/Installment", data)
	end

	def self.createVariablePlan(data)
		if ! data.has_key?("customerId".to_sym)
			return @util.error("customerId is required")
		end
		return @util.post("/Customers/#{customerId}/PaymentSchedules/Variable", data)
	end

	def self.createRecurringPlan(data)
		if ! data.has_key?("customerId".to_sym)
			return @util.error("customerId is required")
		end
		return @util.post("/Customers/#{customerId}/PaymentSchedules/Recurring", data)
	end

	def self.getPlan(data)
		if ! data.has_key?("customerId".to_sym)
			return @util.error("customerId is required")
		end
		if ! data.has_key?("planId".to_sym)
			return @util.error("planId is required")
		end
		return @util.get("/Customers/#{data[:customerId]}/PaymentSchedules/#{data[:planId]}", data)
	end

	def self.deletePlan(data)
		if ! data.has_key?("customerId".to_sym)
			return @util.error("customerId is required")
		end
		if ! data.has_key?("planId".to_sym)
			return @util.error("planId is required")
		end
		return @util.delete("/Customers/#{data[:customerId]}/PaymentSchedules/#{data[:planId]}", data)
	end

	################################################################################
	## Credits
	################################################################################

	def self.credit(data)
		return @util.post("/Payments/Credit", data)
	end

	################################################################################
	## Refunds and Voids
	################################################################################

	def self.refund(data)
		if data.class == Hash
			return @util.post("/Payments/Refund", data)
		else
			return @util.post("/Payments/Refund", { :transactionId => data })
		end
	end

	def self.void(data)
		if data.class == Hash
			return @util.post("/Payments/Void", data)
		else
			return @util.post("/Payments/Void", { :transactionId => data })
		end
	end

	################################################################################
	## Transactions
	################################################################################

	def self.getTransactions(data)
		if data.class == Hash
			return @util.post("/Transactions/Search", data)
		else
			return @util.get("/Transactions/#{data}", nil)
		end
	end

	def self.getTransaction(data)
		if data.class == Hash
			if ! data.has_key?("transactionId".to_sym)
				return @util.error("transactionId is required")
			end
			return @util.get("/Transactions/#{data[:transactionId]}", nil)
		else
			return @util.get("/Transactions/#{data}", nil)
		end
	end

	def self.updateTransaction(data)
		if ! data.has_key?("referenceTransactionId".to_sym)
			return @util.error("referenceTransactionId is required")
		end
		return @util.put("/Transactions/#{data[:referenceTransactionId]}", data)
	end

end
